@@ -25,6 +25,10 @@
 #include <limits.h>
 #include <fnmatch.h>
 
+#ifndef ffsll
+#define ffsll(x) __builtin_ffsll(x)
+#endif
+
 void trap (void);
 
 #define GF_UNIVERSAL_ANSWER 42    /* :O */
@@ -37,12 +41,15 @@ void trap (void);
 #include "locking.h"
 #include "mem-pool.h"
 #include "compat-uuid.h"
+#include "iatt.h"
 #include "uuid.h"
 #include "libglusterfs-messages.h"
 
 #define STRINGIFY(val) #val
 #define TOSTRING(val) STRINGIFY(val)
 
+#define alloca0(size) ({void *__ptr; __ptr = alloca(size); memset(__ptr, 0, size); __ptr; })
+
 #define min(a,b) ((a)<(b)?(a):(b))
 #define max(a,b) ((a)>(b)?(a):(b))
 #define roof(a,b) ((((a)+(b)-1)/((b)?(b):1))*(b))
@@ -68,6 +75,7 @@ void trap (void);
 
 #define GEOREP "geo-replication"
 #define GHADOOP "glusterfs-hadoop"
+#define GLUSTERD_NAME "glusterd"
 
 #define GF_SELINUX_XATTR_KEY "security.selinux"
 
@@ -78,6 +86,11 @@ void trap (void);
          !strcmp (fs_name, "ext3") || \
          !strcmp (fs_name, "ext4"))
 
+/* process mode definitions */
+#define GF_SERVER_PROCESS   0
+#define GF_CLIENT_PROCESS   1
+#define GF_GLUSTERD_PROCESS 2
+
 /* Defining this here as it is needed by glusterd for setting
  * nfs port in volume status.
  */
@@ -140,7 +153,8 @@ enum _gf_special_pid
 
 enum _gf_xlator_ipc_targets {
         GF_IPC_TARGET_CHANGELOG = 0,
-        GF_IPC_TARGET_CTR = 1
+        GF_IPC_TARGET_CTR = 1,
+        GF_IPC_TARGET_UPCALL = 2
 };
 
 typedef enum _gf_boolean gf_boolean_t;
@@ -196,6 +210,8 @@ struct list_node {
         struct list_head list;
 };
 
+extern char *vol_type_str[];
+
 struct list_node *list_node_add (void *ptr, struct list_head *list);
 struct list_node *list_node_add_order (void *ptr, struct list_head *list,
                                        int (*compare)(struct list_head *,
@@ -203,7 +219,7 @@ struct list_node *list_node_add_order (void *ptr, struct list_head *list,
 void list_node_del (struct list_node *node);
 
 struct dnscache *gf_dnscache_init (time_t ttl);
-struct dnscache_entry *gf_dnscache_entry_init ();
+struct dnscache_entry *gf_dnscache_entry_init (void);
 void gf_dnscache_entry_deinit (struct dnscache_entry *entry);
 char *gf_rev_dns_lookup_cached (const char *ip, struct dnscache *dnscache);
 
@@ -216,7 +232,7 @@ int32_t gf_resolve_ip6 (const char *hostname, uint16_t port, int family,
 
 void gf_log_dump_graph (FILE *specfp, glusterfs_graph_t *graph);
 void gf_print_trace (int32_t signal, glusterfs_ctx_t *ctx);
-int  gf_set_log_file_path (cmd_args_t *cmd_args);
+int  gf_set_log_file_path (cmd_args_t *cmd_args, glusterfs_ctx_t *ctx);
 int  gf_set_log_ident (cmd_args_t *cmd_args);
 
 #define VECTORSIZE(count) (count * (sizeof (struct iovec)))
@@ -389,15 +405,15 @@ union gf_sock_union {
 
 #define IOV_MIN(n) min(IOV_MAX,n)
 
-#define GF_FOR_EACH_ENTRY_IN_DIR(entry, dir) \
+#define GF_FOR_EACH_ENTRY_IN_DIR(entry, dir, scr) \
         do {\
                 entry = NULL;\
                 if (dir) { \
-                        entry = sys_readdir (dir); \
+                        entry = sys_readdir (dir, scr); \
                         while (entry && (!strcmp (entry->d_name, ".") || \
                             !fnmatch ("*.tmp", entry->d_name, 0) || \
                             !strcmp (entry->d_name, ".."))) { \
-                                entry = sys_readdir (dir); \
+                                entry = sys_readdir (dir, scr); \
                         } \
                 } \
         } while (0)
@@ -645,6 +661,12 @@ gf_time_fmt (char *dst, size_t sz_dst, time_t utime, unsigned int fmt)
         return dst;
 }
 
+/* This function helps us use gfid (unique identity) to generate inode's unique
+ * number in glusterfs.
+ */
+ino_t
+gfid_to_ino (uuid_t gfid);
+
 int
 mkdir_p (char *path, mode_t mode, gf_boolean_t allow_symlinks);
 /*
@@ -746,15 +768,16 @@ void gf_array_insertionsort (void *a, int l, int r, size_t elem_size,
 int gf_is_str_int (const char *value);
 
 char *gf_uint64_2human_readable (uint64_t);
+int get_vol_type (int type, int dist_count, int brick_count);
 int validate_brick_name (char *brick);
 char *get_host_name (char *word, char **host);
 char *get_path_name (char *word, char **path);
 void gf_path_strip_trailing_slashes (char *path);
-uint64_t get_mem_size ();
+uint64_t get_mem_size (void);
 int gf_strip_whitespace (char *str, int len);
 int gf_canonicalize_path (char *path);
 char *generate_glusterfs_ctx_id (void);
-char *gf_get_reserved_ports();
+char *gf_get_reserved_ports(void);
 int gf_process_reserved_ports (gf_boolean_t ports[], uint32_t ceiling);
 gf_boolean_t
 gf_ports_reserved (char *blocked_port, gf_boolean_t *ports, uint32_t ceiling);
@@ -766,6 +789,9 @@ int gf_set_timestamp  (const char *src, const char* dest);
 
 int gf_thread_create (pthread_t *thread, const pthread_attr_t *attr,
                       void *(*start_routine)(void *), void *arg);
+int gf_thread_create_detached (pthread_t *thread,
+                      void *(*start_routine)(void *), void *arg);
+
 gf_boolean_t
 gf_is_service_running (char *pidfile, int *pid);
 int
@@ -826,4 +852,16 @@ gf_is_zero_filled_stat (struct iatt *buf);
 void
 gf_zero_fill_stat (struct iatt *buf);
 
+gf_boolean_t
+gf_is_valid_xattr_namespace (char *k);
+
+const char *
+gf_inode_type_to_str (ia_type_t type);
+
+int32_t
+gf_bits_count (uint64_t n);
+
+int32_t
+gf_bits_index (uint64_t n);
+
 #endif /* _COMMON_UTILS_H */
